﻿
using System;
using System.ComponentModel;
using System.IO;
using Styx.Helpers;

using DefaultValue = Styx.Helpers.DefaultValueAttribute;
using Styx;
using Styx.CommonBot;
using Singular.Managers;


namespace Singular.Settings
{
    public enum WarlockPet
    {
        None        = 0,        
        Auto        = 1,
        Imp         = 23,       // Pet.CreatureFamily.Id
        Voidwalker  = 16,
        Succubus    = 17,
        Felhunter   = 15,
        Felguard    = 29,
        Other       = 99999     // a quest or other pet forced upon us for some reason
    }

    public enum Soulstone
    {
        None = 0,
        Auto,
        Self,
        Ressurect
    }

    internal class WarlockSettings : Styx.Helpers.Settings
    {

        public WarlockSettings()
            : base(Path.Combine(SingularSettings.SingularSettingsPath, "Warlock.xml"))
        {
        }

        [Setting]
        [DefaultValue(WarlockPet.Auto)]
        [Category("宠物")]
        [DisplayName("召唤宠物")]
        [Description("Auto: 自动选择最适合的宠物.")]
        public WarlockPet Pet { get; set; }

        [Setting]
        [DefaultValue(40)]
        [Category("宠物")]
        [DisplayName("生命通道宠物血量百分比")]
        [Description("战斗中宠物血量低于该百分比时使用生命通道")]
        public int HealthFunnelCast { get; set; }

        [Setting]
        [DefaultValue(95)]
        [Category("宠物")]
        [DisplayName("取消生命通道宠物血量百分比")]
        [Description("战斗中宠物血量高于该百分比时取消正在使用的生命通道")]
        public int HealthFunnelCancel { get; set; }

        [Setting]
        [DefaultValue(75)]
        [Category("宠物")]
        [DisplayName("生命通道宠物血量百分比(休息时)")]
        [Description("休息时宠物血量低于该百分比时使用生命通道")]
        public int HealthFunnelRest { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("宠物")]
        [DisplayName("使用缴械")]
        [Description("True:一冷却就使用缴械; False:不使用")]
        public bool UseDisarm { get; set; }

        [Setting]
        [DefaultValue(20)]
        [Category("常规")]
        [DisplayName("生命吸取血量百分比")]
        [Description("血量低于该百分比时使用生命吸取")]
        public int DrainLifePercentage { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("常规")]
        [DisplayName("使用恐惧")]
        [Description("当血量过低或者控制ADD怪时使用恐惧")]
        public bool UseFear { get; set; }

        [Setting]
        [DefaultValue(3)]
        [Category("常规")]
        [DisplayName("使用恐惧怪物数量")]
        [Description("当攻击术士的怪物数量(不含宠物)达到该数值时使用恐惧; 设置为0禁用数量检查")]
        public int UseFearCount { get; set; }

        [Setting]
        [DefaultValue(Soulstone.Auto)]
        [Category("常规")]
        [DisplayName("使用灵魂石")]
        [Description("Auto: 副本中=复活, 普通/战场=自己, 禁止移动=不使用 -- 复活需要在Singular的战斗设置中先设置好")]
        public Soulstone UseSoulstone { get; set; }

        [Setting]
        [DefaultValue(750)]
        [Category("恶魔")]
        [DisplayName("切换至人形态的恶魔之怒数值")]
        [Description("当恶魔之怒低于(0 - 1000)切换至人形态")]
        public int FurySwitchToCaster { get; set; }

        [Setting]
        [DefaultValue(900)]
        [Category("恶魔")]
        [DisplayName("切换至恶魔形态的恶魔之怒")]
        [Description("当恶魔之怒高于(0 - 1000)切换至恶魔形态")]
        public int FurySwitchToDemon { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("恶魔")]
        [DisplayName("使用恶魔之跃")]
        [Description("使用恶魔之跃来摆脱近战")]
        public bool UseDemonicLeap { get; set; }

        public enum SpellPriority
        {
            Noxxic = 1,
            IcyVeins = 2
        }

        [Setting]
        [DefaultValue(SpellPriority.Noxxic )]
        [Category("毁灭")]
        [DisplayName("法术优先选择")]
        public SpellPriority DestructionSpellPriority { get; set; }



#region Setting Helpers


#endregion

    }
}