﻿
using System.ComponentModel;
using System.IO;
using Styx.Helpers;

using DefaultValue = Styx.Helpers.DefaultValueAttribute;

namespace Singular.Settings
{
    public enum WarriorStance
    {
        Auto,
        BattleStance        = Styx.ShapeshiftForm.BattleStance ,
        BerserkerStance     = Styx.ShapeshiftForm.BerserkerStance ,
        DefensiveStance     = Styx.ShapeshiftForm.DefensiveStance 
    }

    public enum WarriorShout
    {
        CommandingShout,
        BattleShout
    }

    internal class WarriorSettings : Styx.Helpers.Settings
    {
        public WarriorSettings()
            : base(Path.Combine(SingularSettings.SingularSettingsPath, "Warrior.xml"))
        {
        }

        #region Protection

        public enum SpellPriority
        {
            Noxxic = 1,
            IcyVeins = 2,
            ElitistJerks = 3
        }

        [Setting]
        [DefaultValue(SpellPriority.Noxxic)]
        [Category("武器")]
        [DisplayName("法术优先选择")]
        public SpellPriority ArmsSpellPriority { get; set; }

        [Setting]
        [DefaultValue(50)]
        [Category("防护")]
        [DisplayName("狂怒回复血量百分比")]
        [Description("血量低于该百分比时使用狂怒回复")]
        public int WarriorEnragedRegenerationHealth { get; set; }

        [Setting]
        [DefaultValue(30)]
        [Category("防护")]
        [DisplayName("盾墙血量百分比")]
        [Description("血量低于该百分比时使用盾墙")]
        public int WarriorShieldWallHealth { get; set; }


        [Setting]
        [DefaultValue(20)]
        [Category("防护")]
        [DisplayName("破釜沉舟血量百分比")]
        [Description("血量低于该百分比时使用破釜沉舟")]
        public int WarriorLastStandHealth  { get; set; }

        [Setting]
        [DefaultValue(50)]
        [Category("防护")]
        [DisplayName("盾牌格挡血量百分比")]
        [Description("血量低于该百分比时使用盾牌格挡")]
        public int WarriorShieldBlockHealth { get; set; }

        [Setting]
        [DefaultValue(80)]
        [Category("防护")]
        [DisplayName("盾牌屏障血量百分比")]
        [Description("血量低于该百分比时使用盾牌屏障")]
        public int WarriorShieldBarrierHealth { get; set; }

        #endregion

        #region DPS

        [Setting]
        [DefaultValue(true)]
        [Category("输出")]
        [DisplayName("使用打断")]
        [Description("True / False：设置是否让战斗模块进行施法打断")]
        public bool UseWarriorInterrupts { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("输出")]
        [DisplayName("减速")]
        [Description("True / False：设置是否让战斗模块施放减速技能.如:断筋,刺耳怒吼")]
        public bool UseWarriorSlows { get; set; }
        
        [Setting]
        [DefaultValue(true)]
        [Category("输出")]
        [DisplayName("使用冲锋/英勇飞跃?")]
        [Description("True / False：设置是否让战斗模块使用这些快速接近目标的技能")]
        public bool UseWarriorCloser { get; set; }
        #endregion

        [Setting]
        [DefaultValue(WarriorStance.Auto)]
        [Category("输出")]
        [DisplayName("战士输出姿态")]
        [Description("输出时使用的姿态.预料到有伤害来临时会使用战斗姿态否则一直使用狂暴姿态.防护专精则会一直使用防护姿态.")]
        public WarriorStance Stance { get; set; }

        [Setting]
        [DefaultValue(WarriorShout.BattleShout)]
        [Category("常规")]
        [DisplayName("战士怒吼")]
        [Description("怒吼用于保持身上的增益状态,或者在低怒气的情况下使用.CommandingShout:命令怒吼;BattleShout:战斗怒吼")]
        public WarriorShout Shout { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("常规")]
        [DisplayName("乘胜追击")]
        [Description("True: 无视血量百分比,只要乘胜追击/胜利在望冷却一到就用")]
        public bool VictoryRushOnCooldown { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("常规")]
        [DisplayName("使用缴械")]
        [Description("True: 缴械冷却一到就用; False: 不使用该技能")]
        public bool UseDisarm { get; set; }

    }
}